@@ -1,6 +1,8 @@
 """Constants used by ansible_compat."""
 
+from pathlib import Path
 
+META_MAIN = (Path("meta") / Path("main.yml"), Path("meta") / Path("main.yaml"))
 REQUIREMENT_LOCATIONS = [
     "requirements.yml",
     "roles/requirements.yml",
@@ -12,7 +14,7 @@ REQUIREMENT_LOCATIONS = [
 ]
 
 # Minimal version of Ansible we support for runtime
-ANSIBLE_MIN_VERSION = "2.12"
+ANSIBLE_MIN_VERSION = "2.14"
 
 # Based on https://docs.ansible.com/ansible/latest/reference_appendices/config.html
 ANSIBLE_DEFAULT_ROLES_PATH = (
